@@ -53,8 +53,9 @@
 /* Interface name cache */
 char *ifname = NULL;
 
-/* Save sender because scan request is asynchronous */
-static char *sender_cache = NULL;
+/* Save DBUS names */
+static char *scan_name_cache = NULL;
+static char *connect_name_cache = NULL;
 
 /* Selected SSID is saved */
 static gchar *selected_ssid = NULL;
@@ -63,34 +64,21 @@ static gchar *selected_ssid = NULL;
 static gchar *scan_ssid = NULL;
 
 gchar own_mac[ETH_ALEN];
+static int wlan_socket = -1;
 
 static struct wlan_status_t wlan_status;
 static gboolean _flight_mode = FALSE;
 static gboolean _cover_closed = FALSE;
+static gboolean power_down_after_scan = FALSE;
+static gboolean scan_threshold_supported = FALSE;
 static dbus_bool_t saved_inactivity = FALSE;
 
 /* Timer IDs */
 static guint wlan_if_down_timer_id = 0;
 static guint wlan_connect_timer_id = 0;
 
-/* These are for checking correct message parameters */
-
-static const char settings_and_connect_sgn[] = { 
-        DBUS_TYPE_INT32,
-        DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, 
-        DBUS_TYPE_INT32, DBUS_TYPE_INT32,
-        DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE,
-        DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE,
-        DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE,
-        DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE,
-        DBUS_TYPE_INT32,
-        DBUS_TYPE_UINT32,
-        DBUS_TYPE_INVALID };
-
-static const char scan_sgn[] = { 
-        DBUS_TYPE_INT32,
-        DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE,
-        DBUS_TYPE_INVALID };
+/* This is the desired powersave and can be set from DBUS when connecting */
+static guint powersave = WLANCOND_SHORT_CAM;
 
 #ifdef USE_MCE_COVER
 static const char ignore_cover_sgn[] = { DBUS_TYPE_BOOLEAN,
@@ -105,14 +93,15 @@ static gboolean ignore_cover_events = FALSE;
  */
 int socket_open(void) 
 {
-        int sock;
+        if (wlan_socket > 0)
+                return wlan_socket;
         
-        sock = socket(AF_INET, SOCK_DGRAM, 0);
+        wlan_socket = socket(AF_INET, SOCK_DGRAM, 0);
 
-        if (sock < 0)
-                DLOG_ERR("socket() failed");
+        if (wlan_socket < 0)
+                die("socket() failed");
         
-        return sock;
+        return wlan_socket;
 }
 /**
  * Helper function for initializing the iwreq
@@ -185,28 +174,195 @@ static int get_own_mac(void)
         struct ifreq req;
         int sock;
         
-        /* Try to open socket */
-        if ((sock = socket_open()) <0)
-        {
-                return -1;
-        }
+        sock = socket_open();
         
         memset(&req , 0, sizeof(req));
         memcpy(req.ifr_name, ifname, IFNAMSIZ);
         
         if (ioctl(sock, SIOCGIFHWADDR, &req) < 0)
         {
-                close(sock);
                 return -1;
         }
-
-        close(sock);
         
         memcpy(own_mac, req.ifr_hwaddr.sa_data, ETH_ALEN);
         
         return 0;
 }
 
+/** 
+    Helper function for getting Gconf values. 
+    @param path Gconf path to search for user specified value.
+    @return CAM timeout.
+*/
+static gint get_gconf_int(const gchar* path) 
+{
+        GConfClient *client;
+        GConfValue *gconf_value;
+        GError *error = NULL;
+ 
+        client = gconf_client_get_default();
+        if (client == NULL) {
+                return -1;
+        }
+
+        gconf_value = gconf_client_get(client, path, &error); 
+
+        if (error != NULL) {
+                DLOG_ERR("Could not get setting:%s, error:%s", 
+                         SLEEP_GCONF_PATH, 
+                         error->message);
+                
+                g_clear_error(&error);
+                g_object_unref(client);
+                return -1;
+        }
+        g_object_unref(client);
+        
+        if (gconf_value == NULL) {
+                return -1;
+        }
+        if (gconf_value->type == GCONF_VALUE_INT) {
+                gint value = gconf_value_get_int(gconf_value);
+                DLOG_DEBUG("User selected value: %d", value);
+                gconf_value_free(gconf_value);
+                return value;
+        }
+        
+        gconf_value_free(gconf_value);
+        return -1;
+}
+
+static void set_dma_threshold(void) 
+{
+        struct iwreq req;
+        int sock, dma_threshold;
+
+        init_iwreq(&req);
+
+        sock = socket_open();
+
+        dma_threshold = get_gconf_int(DMA_THRESHOLD_GCONF_PATH);
+
+        if (dma_threshold < 0)
+                dma_threshold = WLANCOND_DEFAULT_DMA_THRESHOLD;
+
+        req.u.mode = dma_threshold;
+        
+        if (ioctl(sock, SIOCIWFIRSTPRIV + 20, &req) < 0) {
+                DLOG_ERR("set DMA failed");
+                return;
+        }
+        //DLOG_DEBUG("Set DMA threshold to %d", dma_threshold);
+}
+
+#define MCE_DEVLOCK_FILENAME "/var/run/mce/call"
+/** 
+    Check if call is going 
+    @return TRUE if call going.
+*/
+static gboolean call_going(void) 
+{
+        GIOChannel *iochan = NULL;
+	GIOStatus iostatus;
+	GError *error = NULL;
+        gchar* buf;
+        gboolean ret = TRUE;
+
+	if ((iochan = g_io_channel_new_file(MCE_DEVLOCK_FILENAME,
+					    "r", &error)) == NULL) {
+		DLOG_DEBUG("Cannot open for reading: %s", error->message);
+		g_clear_error(&error);
+                
+		return ret;
+	}
+        
+	g_clear_error(&error);
+        
+        buf = g_malloc0(4);
+        
+        iostatus = g_io_channel_read_chars(iochan, buf, sizeof(buf),
+                                           NULL, &error);
+        
+	if (iostatus != G_IO_STATUS_NORMAL)
+		DLOG_DEBUG("Cannot read: %s", error->message);
+        
+	g_clear_error(&error);
+        
+	iostatus = g_io_channel_shutdown(iochan, TRUE, &error);
+        
+	if (iostatus != G_IO_STATUS_NORMAL)
+		DLOG_DEBUG("Cannot close: %s", error->message);
+        
+        if (strncmp(buf, "no", 2) == 0)
+        {
+                ret = FALSE;
+        }
+        
+        g_free(buf);
+	g_clear_error(&error);
+	g_io_channel_unref(iochan);
+        
+	return ret;
+}
+
+static void set_bgscan_params(gboolean activity) 
+{
+        struct iwreq req;
+        int sock, bgscan_threshold;
+        
+        if (scan_threshold_supported == FALSE)
+                return;
+        
+        init_iwreq(&req);
+
+        sock = socket_open();
+
+        bgscan_threshold = get_gconf_int(BGSCAN_THRESHOLD_GCONF_PATH);
+
+        if (bgscan_threshold < 0) {
+                if (activity == TRUE && call_going() == FALSE)
+                        bgscan_threshold = WLANCOND_DEFAULT_IDLE_BGSCAN_THRESHOLD;
+                else
+                        bgscan_threshold = WLANCOND_DEFAULT_BGSCAN_THRESHOLD;
+        }
+                
+        req.u.mode = -bgscan_threshold;
+        
+        if (ioctl(sock, SIOCIWFIRSTPRIV + 4, &req) < 0) {
+                DLOG_ERR("set bgscan failed");
+        }
+        //DLOG_DEBUG("Set bgscan_threshold to %d", -bgscan_threshold);
+
+        return;
+}
+
+static void set_bgscan_interval(void) 
+{
+        struct iwreq req;
+        int sock, bgscan_interval;
+        
+        if (scan_threshold_supported == FALSE)
+                return;
+
+        bgscan_interval = get_gconf_int(BGSCAN_INTERVAL_GCONF_PATH);
+        
+        if (bgscan_interval < 0 || bgscan_interval >= G_MAXUINT16) {
+                // No user set interval, don't do anything
+                return;
+        }
+
+        init_iwreq(&req);
+
+        sock = socket_open();
+        
+        req.u.mode = bgscan_interval;
+        
+        if (ioctl(sock, SIOCIWFIRSTPRIV + 2, &req) < 0) {
+                DLOG_ERR("set bgscan failed");
+        }
+        DLOG_DEBUG("Set bgscan_interval to %d", bgscan_interval);
+}
+
 void update_own_ie(unsigned char* wpa_ie, guint wpa_ie_len) 
 {
         if (wlan_status.wpa_ie.ie != NULL) {
@@ -238,12 +394,14 @@ int get_encryption_info(void)
         return auth_status;
 }
 
-
 /**
  * Helper function for initializing handler structs
  */
 int init_dbus_handler(void)
 {
+        int sock, count, i;
+        iwprivargs *priv;
+        
         memset(&wlan_status, 0, sizeof(wlan_status));
 
         if (get_own_mac() < 0) {
@@ -251,6 +409,26 @@ int init_dbus_handler(void)
                 return -1; 
         }
 
+        set_dma_threshold();
+
+        sock = socket_open();
+
+        count = iw_get_priv_info(sock, ifname, &priv);
+
+        if (count > 0)
+        {
+                for (i = 0; i < count; i++)
+                        if (priv[i].name[0] != '\0') {
+                                if (strcmp(priv[i].name, "set_scanthres") == 0) 
+                                {
+                                        scan_threshold_supported = TRUE;
+                                        break;
+                                }
+                        }
+        }
+        if (priv)
+                free(priv);
+        
         return 0;
 }
 /**
@@ -260,6 +438,8 @@ int clean_dbus_handler(void)
 {
         if (ifname != NULL)
                 g_free(ifname);
+        if (wlan_socket > 0)
+                close(wlan_socket);
         return 0;
 }
 /**
@@ -324,24 +504,24 @@ static DBusHandlerResult activity_check_dbus(DBusMessage *message) {
 void activity_check(dbus_bool_t inactivity) {
 
         int sock;
-
+        
         if (get_wlan_state() != WLAN_CONNECTED) {
                 return;
         }
         
-        if ((sock = socket_open()) <0)
-        {
-                return;
-        }
+        sock = socket_open();
+
         if (inactivity == FALSE) {
                 //DLOG_DEBUG("WLAN activity mode changed to active");
-                set_power_state(WLANCOND_SHORT_CAM, sock);
+                set_bgscan_params(FALSE);
         } else {
                 //DLOG_DEBUG("WLAN activity mode changed to inactive");
-                set_power_state(WLANCOND_VERY_SHORT_CAM, sock);
+                set_bgscan_params(TRUE);
         }
-        close(sock);
+
+        set_power_state(powersave, sock);     
 }
+
 /**
  * Helper function for determining inactivity
  */
@@ -357,7 +537,6 @@ static DBusHandlerResult icd_check_signal_dbus(DBusMessage *message) {
         char *icd_type;
         char *icd_state;
         char *icd_disconnect_reason;
-        guint ps_mode = WLANCOND_SHORT_CAM;
         DBusError dbus_error;
         int sock;
 
@@ -384,18 +563,13 @@ static DBusHandlerResult icd_check_signal_dbus(DBusMessage *message) {
         if (icd_state != NULL && strncmp(icd_state, "CONNECTED", 9) == 0) {
 
                 set_wlan_state(WLAN_CONNECTED, NO_SIGNAL, FORCE_NO);
-
-                if ((sock = socket_open()) >= 0) {
+                
+                sock = socket_open();
                         
-                        DLOG_DEBUG("Going to power save");
-
-                        if (get_inactivity_status() == TRUE)
-                                ps_mode = WLANCOND_VERY_SHORT_CAM;
+                DLOG_DEBUG("Going to power save");
                         
-                        if (set_power_state(ps_mode, sock) == FALSE) {
-                                DLOG_ERR("Failed to set power save");
-                        }
-                        close(sock);
+                if (set_power_state(powersave, sock) == FALSE) {
+                        DLOG_ERR("Failed to set power save");
                 }
         }
         //DLOG_DEBUG("Handled icd signal, icd_state:%s", icd_state);
@@ -508,10 +682,7 @@ void set_wlan_state(int new_state, int send_signal, force_t force)
                         }
                 }               
                 
-                if ((sock = socket_open()) <0)
-                {
-                        return;
-                }
+                sock = socket_open();
                 
                 if (force == FORCE_YES) {
 
@@ -522,7 +693,6 @@ void set_wlan_state(int new_state, int send_signal, force_t force)
                         }
                         
                         if (set_interface_state(sock, CLEAR, IFF_UP)<0) {
-                                close(sock);
                                 DLOG_ERR("Could not set interface down");
                                 return;
                         }
@@ -531,7 +701,9 @@ void set_wlan_state(int new_state, int send_signal, force_t force)
                         
                         DLOG_DEBUG("Delaying interface shutdown");
                         
-                        set_power_state(WLANCOND_FULL_POWERSAVE, sock);
+                        /* Removed, does not reduce power consumption
+                           when not connected */
+                        //set_power_state(WLANCOND_FULL_POWERSAVE, sock);
 
                         if (wlan_if_down_timer_id != 0) {
                                 DLOG_DEBUG("Already delaying...");
@@ -545,8 +717,6 @@ void set_wlan_state(int new_state, int send_signal, force_t force)
                         
                 }
                 
-                close(sock);
-
                 if (send_signal == DISCONNECTED_SIGNAL)
                         disconnected_signal();       
                 
@@ -575,11 +745,11 @@ void set_scan_state(guint new_state)
                 return;
         }
         if (new_state == SCAN_NOT_ACTIVE && wlan_status.scan == SCAN_ACTIVE && 
-            sender_cache != NULL) {
+            scan_name_cache != NULL) {
                 DLOG_DEBUG("Sending empty results");
-                send_dbus_scan_results(NULL, sender_cache);
-                g_free(sender_cache);
-                sender_cache = NULL;
+                send_dbus_scan_results(NULL, scan_name_cache, 0);
+                g_free(scan_name_cache);
+                scan_name_cache = NULL;
         }
         
         DLOG_DEBUG("Wlancond scan change, old_state: %s, new_state: %s", 
@@ -605,50 +775,6 @@ int get_mode(void)
         return wlan_status.mode;
 }
 
-/** 
-    Helper function for resolving CAM timeout. 
-    @param path Gconf path to search for user specified value.
-    @return CAM timeout.
-*/
-static gint get_sleep_timeout(const gchar* path) 
-{
-        GConfClient *client;
-        GConfValue *gconf_value;
-        GError *error = NULL;
- 
-        client = gconf_client_get_default();
-        if (client == NULL) {
-                return -1;
-        }
-
-        gconf_value = gconf_client_get(client, path, &error); 
-
-        if (error != NULL) {
-                DLOG_ERR("Could not get setting:%s, error:%s", 
-                         SLEEP_GCONF_PATH, 
-                         error->message);
-                
-                g_clear_error(&error);
-                g_object_unref(client);
-                return -1;
-        }
-        g_object_unref(client);
-        
-        if (gconf_value == NULL) {
-                return -1;
-        }
-        if (gconf_value->type == GCONF_VALUE_INT) {
-                gint sleep_timeout = gconf_value_get_int(gconf_value);
-                DLOG_DEBUG("User selected sleep timeout: %d ms", 
-                           sleep_timeout);
-                gconf_value_free(gconf_value);
-                return sleep_timeout;
-        }
-        
-        gconf_value_free(gconf_value);
-        return -1;
-}
-
 /**
    Set WLAN power state.
    @param state New power state.
@@ -660,10 +786,13 @@ gboolean set_power_state(guint new_state, int sock)
         struct iwreq req;
         gint sleep_timeout;
         
+        if (new_state == WLANCOND_SHORT_CAM && get_inactivity_status() == TRUE)
+                new_state = WLANCOND_VERY_SHORT_CAM;
+        
         if (wlan_status.power == new_state) {
                 return TRUE;
         }
-        
+
         init_iwreq(&req);
 
         switch (new_state) {
@@ -676,14 +805,14 @@ gboolean set_power_state(guint new_state, int sock)
                     break;
             case WLANCOND_SHORT_CAM:
                     req.u.power.flags = IW_POWER_MULTICAST_R;
-                    sleep_timeout = get_sleep_timeout(SLEEP_GCONF_PATH);
+                    sleep_timeout = get_gconf_int(SLEEP_GCONF_PATH);
                     if (sleep_timeout < 0)
                             sleep_timeout = WLANCOND_DEFAULT_SLEEP_TIMEOUT;
                     req.u.power.value = sleep_timeout;
                     break;
             case WLANCOND_VERY_SHORT_CAM:
                     req.u.power.flags = IW_POWER_MULTICAST_R;
-                    sleep_timeout = get_sleep_timeout(INACTIVE_SLEEP_GCONF_PATH);
+                    sleep_timeout = get_gconf_int(INACTIVE_SLEEP_GCONF_PATH);
                     if (sleep_timeout < 0)
                             sleep_timeout = WLANCOND_VERY_SHORT_CAM_TIMEOUT;
                     req.u.power.value = sleep_timeout;
@@ -714,9 +843,13 @@ static int init_if(int sock)
 {
         int previous_state = get_wlan_state();
         
-        if (get_wlan_state() == WLAN_NOT_INITIALIZED) {
-                if (set_interface_state(sock, SET, IFF_UP | IFF_RUNNING)<0) {
-                        return -1;
+        if (previous_state == WLAN_NOT_INITIALIZED) {
+                /* Check if interface is still up from delayed shutdown */
+                if (wlan_if_down_timer_id == 0) {
+                        if (set_interface_state(sock, SET, 
+                                                IFF_UP | IFF_RUNNING) < 0) {
+                                return -1;
+                        }
                 }
                 set_power_state(WLANCOND_POWER_ON, sock);
                 set_wlan_state(WLAN_INITIALIZED, NO_SIGNAL, FORCE_YES);
@@ -745,6 +878,8 @@ static int set_we_name(int sock, char *name, char *args[], int count)
                            name);
         } else {
                 DLOG_DEBUG("Found interface %s", name);
+                if (ifname != NULL)
+                        g_free(ifname);
                 ifname = g_malloc(IFNAMSIZ+1);
                 strncpy(ifname, name, IFNAMSIZ);
                 ifname[IFNAMSIZ] = '\0';
@@ -761,13 +896,10 @@ int get_we_device_name(void)
 {
         int sock;
         
-        if ((sock = socket_open()) <0)
-                return -1;
+        sock = socket_open();
         
         iw_enum_devices(sock, &set_we_name, NULL, 0);
 
-        close (sock);
-        
         if (strnlen(ifname, IFNAMSIZ) < 2)
                 return -1;
 
@@ -955,6 +1087,7 @@ static DBusHandlerResult settings_and_connect_request(
         struct iwreq req;
         dbus_int32_t mode, encryption, power_level, default_key;
         dbus_uint32_t adhoc_channel = 0;
+        dbus_uint32_t flags = 0;
         char *ssid;
         unsigned char* key[4];
         int key_len[4];
@@ -965,13 +1098,6 @@ static DBusHandlerResult settings_and_connect_request(
 
 	dbus_error_init(&derror);
 
-        if (!dbus_message_has_signature(message, settings_and_connect_sgn)) {
-                if (send_invalid_args(connection, message) < 0) {
-                        DLOG_ERR("sending reply failed");
-                }
-                return DBUS_HANDLER_RESULT_HANDLED;
-        }
-
         if (in_flight_mode() || cover_closed()) {
                 reply = new_dbus_error(message, WLANCOND_ERROR_WLAN_DISABLED);
                 send_and_unref(connection, reply);
@@ -980,12 +1106,7 @@ static DBusHandlerResult settings_and_connect_request(
 
         remove_connect_timer();
         
-        if ((sock = socket_open()) <0)
-        {
-                reply = new_dbus_error(message, WLANCOND_ERROR_FATAL);
-                send_and_unref(connection, reply);
-                return DBUS_HANDLER_RESULT_HANDLED;
-        }
+        sock = socket_open();
 
         if ((previous_state = init_if(sock)) < 0) {
                 reply = new_dbus_error(message, WLANCOND_ERROR_INIT_FAILED);
@@ -996,8 +1117,10 @@ static DBusHandlerResult settings_and_connect_request(
         
         set_power_state(WLANCOND_POWER_ON, sock);
 
+        set_bgscan_interval();
+
 	if (dbus_message_get_args(
-                    message, &derror,        
+                    message, NULL,        
                     DBUS_TYPE_INT32, &power_level,
                     DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, &ssid, &ssid_len,
                     DBUS_TYPE_INT32, &mode,
@@ -1008,12 +1131,46 @@ static DBusHandlerResult settings_and_connect_request(
                     DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, &key[3], &key_len[3],
                     DBUS_TYPE_INT32, &default_key,
                     DBUS_TYPE_UINT32, &adhoc_channel,
+                    DBUS_TYPE_UINT32, &flags,
                     DBUS_TYPE_INVALID) == FALSE) 
         {
-                DLOG_ERR("Failed to parse setting_and_connect: %s",
-			 derror.message);
-		dbus_error_free(&derror);
-                goto param_err;
+                /* Try without flags */
+                if (dbus_message_get_args(
+                            message, &derror,        
+                            DBUS_TYPE_INT32, &power_level,
+                            DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, &ssid, &ssid_len,
+                            DBUS_TYPE_INT32, &mode,
+                            DBUS_TYPE_INT32, &encryption,
+                            DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, 
+                            &key[0], &key_len[0],
+                            DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, 
+                            &key[1], &key_len[1],
+                            DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, 
+                            &key[2], &key_len[2],
+                            DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, 
+                            &key[3], &key_len[3],
+                            DBUS_TYPE_INT32, &default_key,
+                            DBUS_TYPE_UINT32, &adhoc_channel,
+                            DBUS_TYPE_INVALID) == FALSE) {
+
+                        DLOG_ERR("Failed to parse setting_and_connect: %s",
+                                 derror.message);
+                        dbus_error_free(&derror);
+                        goto param_err;
+                }
+        }
+
+        if (flags & WLANCOND_DISABLE_POWERSAVE) {
+                DLOG_DEBUG("Powersave disabled");
+                powersave = WLANCOND_POWER_ON;
+        } else if (flags & WLANCOND_MINIMUM_POWERSAVE) {
+                DLOG_DEBUG("Powersave minimum");
+                powersave = WLANCOND_LONG_CAM;
+        } else if (flags & WLANCOND_MAXIMUM_POWERSAVE) {
+                DLOG_DEBUG("Powersave maximum");
+                powersave = WLANCOND_SHORT_CAM;
+        } else {
+                powersave = WLANCOND_SHORT_CAM;
         }
         
         if (power_level != WLANCOND_TX_POWER10 &&
@@ -1172,7 +1329,7 @@ static DBusHandlerResult settings_and_connect_request(
         init_iwreq(&req);
 
         req.u.essid.pointer = (caddr_t)ssid;
-        req.u.essid.length = ssid_len;
+        req.u.essid.length = ssid_len -1; // Remove NULL termination
         req.u.essid.flags = 1 | get_auth_mode(encryption, wpa2_mode);
         
         /* ESSID is set as a last item*/
@@ -1187,9 +1344,11 @@ static DBusHandlerResult settings_and_connect_request(
                         WLANCOND_CONNECT_TIMEOUT, 
                         wlan_connect_timer_cb, NULL);
         }
+        if (connect_name_cache != NULL)
+                g_free(connect_name_cache);
+        
+        connect_name_cache = g_strdup(dbus_message_get_sender(message));
         
-        close (sock);
-
         reply = new_dbus_method_return(message);
         
         append_dbus_args(reply,
@@ -1207,7 +1366,6 @@ static DBusHandlerResult settings_and_connect_request(
                 reply = new_dbus_error(message, DBUS_ERROR_INVALID_ARGS);
         }
         send_and_unref(connection, reply);
-        close (sock);
         return DBUS_HANDLER_RESULT_HANDLED;       
 }
 
@@ -1221,24 +1379,17 @@ static DBusHandlerResult scan_request(DBusMessage    *message,
                                       DBusConnection *connection) {
         
         DBusMessage *reply = NULL;
+        DBusMessageIter iter, array_iter;
         struct iwreq req;
         char *ssid;
         int ssid_len;
         int sock = 0;
         const char* sender;
         dbus_int32_t power_level;
+        dbus_int32_t flags;
         int previous_state = 0;
-        DBusError derror;
-
-	dbus_error_init(&derror);
+        gboolean passive_scan = FALSE;
         
-        if (!dbus_message_has_signature(message, scan_sgn)) {
-                if (send_invalid_args(connection, message) < 0) {
-                        DLOG_ERR("sending reply failed");
-                }
-                return DBUS_HANDLER_RESULT_HANDLED;
-        }
-
         if (in_flight_mode() || cover_closed()) {
                 reply = new_dbus_error(message, WLANCOND_ERROR_WLAN_DISABLED);
                 send_and_unref(connection, reply);
@@ -1259,13 +1410,7 @@ static DBusHandlerResult scan_request(DBusMessage    *message,
                 return DBUS_HANDLER_RESULT_HANDLED;
         }
 
-        /* Try to open socket */
-        if ((sock = socket_open()) <0)
-        {
-                reply = new_dbus_error(message, WLANCOND_ERROR_FATAL);
-                send_and_unref(connection, reply);
-                return DBUS_HANDLER_RESULT_HANDLED;
-        }
+        sock = socket_open();
         
         if ((previous_state = init_if(sock)) < 0) {
                 reply = new_dbus_error(message, WLANCOND_ERROR_INIT_FAILED);
@@ -1274,18 +1419,34 @@ static DBusHandlerResult scan_request(DBusMessage    *message,
         if (previous_state == WLAN_NOT_INITIALIZED) {
                 set_wlan_state(WLAN_INITIALIZED_FOR_SCAN, NO_SIGNAL, FORCE_NO);
         }
+        
+	dbus_message_iter_init(message, &iter);
+        
+	if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_INT32)
+		goto param_err;
+	dbus_message_iter_get_basic(&iter, &power_level);
 
-	if (dbus_message_get_args(
-                    message, &derror,        
-                    DBUS_TYPE_INT32, &power_level,
-                    DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, &ssid, &ssid_len,
-                    DBUS_TYPE_INVALID) == FALSE) 
-        {
-                DLOG_ERR("Failed to parse scan request: %s",
-			 derror.message);
-		dbus_error_free(&derror);
+	dbus_message_iter_next(&iter);
+	
+        if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_ARRAY ||
+            dbus_message_iter_get_element_type(&iter) != DBUS_TYPE_BYTE)
                 goto param_err;
+        dbus_message_iter_recurse(&iter, &array_iter);
+        dbus_message_iter_get_fixed_array(&array_iter, &ssid, &ssid_len);
+        dbus_message_iter_next(&iter);
+
+        power_down_after_scan = FALSE;
+        
+        if (dbus_message_iter_get_arg_type(&iter) == DBUS_TYPE_UINT32) {
+                dbus_message_iter_get_basic(&iter, &flags);
+                DLOG_DEBUG("Found flags: %08x", flags);
+
+                if (flags & WLANCOND_NO_DELAYED_SHUTDOWN)
+                        power_down_after_scan = TRUE;
+                if (flags & WLANCOND_PASSIVE_SCAN)
+                        passive_scan = TRUE;
         }
+
         init_iwreq(&req);
         
         if (power_level != WLANCOND_TX_POWER10 &&
@@ -1300,8 +1461,11 @@ static DBusHandlerResult scan_request(DBusMessage    *message,
         
         init_iwreq(&req);
         req.u.essid.pointer = (caddr_t)ssid;
-        req.u.essid.flags = IW_SCAN_THIS_ESSID;
-
+        if (passive_scan == TRUE)
+                req.u.essid.flags = IW_SCAN_ALL_ESSID;
+        else
+                req.u.essid.flags = IW_SCAN_THIS_ESSID;
+        
         if (ssid_len > WLANCOND_MAX_SSID_SIZE + 1) {
                 DLOG_DEBUG("Invalid SSID\n");
                 goto param_err;
@@ -1309,35 +1473,25 @@ static DBusHandlerResult scan_request(DBusMessage    *message,
         
         if (ssid != NULL && ssid_len > 1) {
                 DLOG_DEBUG("Found ssid '%s' for active scan", ssid);
-                req.u.essid.length = ssid_len;
+                req.u.essid.length = ssid_len -1; // Remove NULL termination
                 
                 if (scan_ssid != NULL)
                         g_free(scan_ssid);
                 scan_ssid = g_strdup(ssid);
         }
         
-#ifdef NORMAL_SCAN
         if (ioctl(sock, SIOCSIWSCAN, &req) < 0) {
                 DLOG_ERR("Scan ioctl failed\n");
                 reply = new_dbus_error(message, WLANCOND_ERROR_IOCTL_FAILED);
                 goto param_err;
         }
-#else
-        if (ioctl(sock, SM_DRV_WPA_SCAN, &req) < 0) {
-                DLOG_ERR("WPA scan ioctl failed\n");
-                reply = new_dbus_error(message, WLANCOND_ERROR_IOCTL_FAILED);
-                goto param_err;
-        }
-#endif
 
-        if (sender_cache != NULL)
-                g_free(sender_cache);
-        sender_cache = g_strdup(sender);
+        if (scan_name_cache != NULL)
+                g_free(scan_name_cache);
+        scan_name_cache = g_strdup(sender);
 
         set_scan_state(SCAN_ACTIVE);
         
-        close (sock);
-        
         reply = new_dbus_method_return(message);
         if (send_and_unref(connection, reply) < 0) {
                 DLOG_ERR("Sending message failed!");
@@ -1354,7 +1508,6 @@ static DBusHandlerResult scan_request(DBusMessage    *message,
                 reply = new_dbus_error(message, DBUS_ERROR_INVALID_ARGS);
         }
         send_and_unref(connection, reply);
-        close (sock);
         return DBUS_HANDLER_RESULT_HANDLED;
 }
 /**  
@@ -1404,19 +1557,16 @@ static int network_compare(gconstpointer a, gconstpointer b)
 int ask_scan_results(int ifindex) 
 {
         struct iwreq req;
-        char *buffer = NULL;
+        char *buffer;
         unsigned int buflen = IW_SCAN_MAX_DATA;
         GSList *scan_results_save = NULL;
+        dbus_int32_t number_of_results;
         int sock;
         unsigned int counter = 3;
         
         init_iwreq(&req);
 
-        /* Try to open socket */
-        if ((sock = socket_open()) <0)
-        {
-                goto param_err_no_free;
-        }
+        sock = socket_open();
         
         buffer = g_malloc(buflen);
         
@@ -1513,38 +1663,43 @@ int ask_scan_results(int ifindex)
                         g_free(scan_results);
                 }
         }
+
+        if (get_wlan_state() == WLAN_INITIALIZED_FOR_SCAN &&
+            power_down_after_scan == TRUE) {  
+                set_interface_state(sock, CLEAR, IFF_UP);
+        }
         
-        if (scan_results_save != NULL)
+        number_of_results = g_slist_length(scan_results_save);
+        
+        /* Sort the list only if the amount of networks is very high and
+         we need to restrict the results */
+        if (number_of_results > WLANCOND_MAX_NETWORKS)
                 scan_results_save = g_slist_sort(scan_results_save, 
                                                  network_compare);
         
-        send_dbus_scan_results(scan_results_save, sender_cache);
+        send_dbus_scan_results(scan_results_save, scan_name_cache, 
+                               number_of_results);
         clean_scan_results(scan_results_save);
         
-        g_free(sender_cache);
-        sender_cache = NULL;
+        g_free(scan_name_cache);
+        scan_name_cache = NULL;
         set_scan_state(SCAN_NOT_ACTIVE);
-        
-        if (get_wlan_state() == WLAN_INITIALIZED_FOR_SCAN) {
+
+        if (get_wlan_state() == WLAN_INITIALIZED_FOR_SCAN)
                 set_wlan_state(WLAN_NOT_INITIALIZED, NO_SIGNAL, FORCE_NO);
-        }
-        close (sock);
+
         g_free(buffer);
 
         return TRUE;
         
   param_err:
-        g_free(buffer);
-        
-  param_err_no_free:
+        g_free(buffer);        
         DLOG_DEBUG("Scan failed");
-        send_dbus_scan_results(scan_results_save, sender_cache);
+        send_dbus_scan_results(scan_results_save, scan_name_cache, 0);
         clean_scan_results(scan_results_save);
-        g_free(sender_cache);
-        sender_cache = NULL;
+        g_free(scan_name_cache);
+        scan_name_cache = NULL;
         set_scan_state(SCAN_NOT_ACTIVE);
-        if (sock >= 0) 
-                close (sock);
         return FALSE;
 }
 
@@ -1589,24 +1744,20 @@ static DBusHandlerResult status_request(DBusMessage    *message,
         int sock;
 
         if (get_wlan_state() != WLAN_CONNECTED && 
-            get_wlan_state() != WLAN_NO_ADDRESS) {
+            get_wlan_state() != WLAN_NO_ADDRESS &&
+            wlan_status.mode != WLANCOND_ADHOC) {
                 reply = new_dbus_error(message, WLANCOND_ERROR_IOCTL_FAILED);
                 send_and_unref(connection, reply);
                 return DBUS_HANDLER_RESULT_HANDLED;
         }
                 
-        if ((sock = socket_open()) < 0)
-        {
-                reply = new_dbus_error(message, WLANCOND_ERROR_FATAL);
-                send_and_unref(connection, reply);
-                return DBUS_HANDLER_RESULT_HANDLED;
-        }
+        sock = socket_open();
 
         init_iwreq(&req);
         
-        essid = g_malloc(IW_ESSID_MAX_SIZE+1);
+        essid = g_malloc0(IW_ESSID_MAX_SIZE+1);
         req.u.essid.pointer = (caddr_t)essid;
-        req.u.essid.length = IW_ESSID_MAX_SIZE + 1;
+        req.u.essid.length = IW_ESSID_MAX_SIZE;
         req.u.essid.flags = 0;
 
         /* essid */
@@ -1617,11 +1768,12 @@ static DBusHandlerResult status_request(DBusMessage    *message,
         }
         essid_len = req.u.essid.length;
 
-        if (essid_len == 0)
-                essid_len = 1;
+        // Handle corner cases to keep the API the same
+        if (essid_len == 0 || essid_len == 32)
+                essid_len++;
         
         init_iwreq(&req);
-
+        
         /* bssid */
         if (ioctl(sock, SIOCGIWAP, &req) < 0) {
                 DLOG_ERR("Could not get BSSID");
@@ -1718,8 +1870,6 @@ static DBusHandlerResult status_request(DBusMessage    *message,
         } 
         capability |= req.u.bitrate.value;
         
-        close(sock);
-        
         reply = new_dbus_method_return(message);
 
         append_dbus_args(reply,
@@ -1753,7 +1903,6 @@ static DBusHandlerResult status_request(DBusMessage    *message,
                 reply = new_dbus_error(message, DBUS_ERROR_INVALID_ARGS);
         }
         send_and_unref(connection, reply);
-        close (sock);
         return DBUS_HANDLER_RESULT_HANDLED;
 }
 
@@ -1831,12 +1980,7 @@ static DBusHandlerResult set_pmksa_request(DBusMessage    *message,
 
 	dbus_error_init(&derror);
 
-        if ((sock = socket_open()) < 0)
-        {
-                reply = new_dbus_error(message, WLANCOND_ERROR_FATAL);
-                send_and_unref(connection, reply);
-                return DBUS_HANDLER_RESULT_HANDLED;
-        }
+        sock = socket_open();
         
         if (dbus_message_get_args(
                     message, &derror,        
@@ -1878,7 +2022,6 @@ static DBusHandlerResult set_pmksa_request(DBusMessage    *message,
                 reply = new_dbus_error(message, WLANCOND_ERROR_IOCTL_FAILED);
                 goto param_err;
         }
-        close(sock);
 
 #ifdef DEBUG        
         char* a = pmksa.bssid.sa_data;
@@ -1898,7 +2041,6 @@ static DBusHandlerResult set_pmksa_request(DBusMessage    *message,
                 reply = new_dbus_error(message, DBUS_ERROR_INVALID_ARGS);
         }
         send_and_unref(connection, reply);
-        close (sock);
         return DBUS_HANDLER_RESULT_HANDLED;
 } 
 
@@ -1915,14 +2057,8 @@ static DBusHandlerResult set_powersave_request(DBusMessage    *message,
         int sock;
         DBusError error;
         dbus_bool_t onoff;
-        guint ps_mode = WLANCOND_SHORT_CAM;
         
-        if ((sock = socket_open()) < 0)
-        {
-                reply = new_dbus_error(message, WLANCOND_ERROR_FATAL);
-                send_and_unref(connection, reply);
-                return DBUS_HANDLER_RESULT_HANDLED;
-        }
+        sock = socket_open();
         
         dbus_error_init(&error);
         
@@ -1933,21 +2069,21 @@ static DBusHandlerResult set_powersave_request(DBusMessage    *message,
                          error.message);
                 dbus_error_free(&error);
                 
-                reply = new_dbus_error(message, DBUS_ERROR_INVALID_ARGS);
-                send_and_unref(connection, reply);
-                close (sock);
+                send_invalid_args(connection, message);
                 return DBUS_HANDLER_RESULT_HANDLED;
         }
         
+        /* Powersave can be allowed only when we are properly connected
+           or when the entity asking for connection wants powersave despite of
+           the state */
         if (onoff == TRUE) {
                 if (get_wlan_state() == WLAN_NOT_INITIALIZED && !get_mic_status()) {
                         set_wlan_state(WLAN_NOT_INITIALIZED, NO_SIGNAL, FORCE_YES);
-                } else if (get_wlan_state() != WLAN_NO_ADDRESS) {
-                        // Go to power save if not acquiring address
-                        if (get_inactivity_status() == TRUE)
-                                ps_mode = WLANCOND_VERY_SHORT_CAM;
-                        if (set_power_state(ps_mode, sock) == 
-                            FALSE) {
+                } else if (get_wlan_state() != WLAN_NO_ADDRESS ||
+                           (connect_name_cache != NULL && 
+                            strcmp(dbus_message_get_sender(message), 
+                                   connect_name_cache) == 0)) {
+                        if (set_power_state(powersave, sock) == FALSE) {
                                 DLOG_ERR("Setting powersave failed");
                                 // Not fatal
                         }
@@ -1960,8 +2096,6 @@ static DBusHandlerResult set_powersave_request(DBusMessage    *message,
                 }
         }
         
-        close(sock);
-        
         DLOG_DEBUG("WLAN powersave %s", onoff==TRUE?"on":"off");
         
         reply = new_dbus_method_return(message);
