@@ -116,7 +116,8 @@ GSList *save_scan_results(struct scan_results_t *scan_results,
     @param scan_results_save structure where scan results are saved.
     @param sender The message is sent to this entity. 
 */     
-void send_dbus_scan_results(GSList *scan_results_save, const char* sender) 
+void send_dbus_scan_results(GSList *scan_results_save, const char* sender,
+                            dbus_int32_t number_of_results) 
 {
         DBusMessage *results;
         DBusMessageIter iter, sub;
@@ -135,9 +136,6 @@ void send_dbus_scan_results(GSList *scan_results_save, const char* sender)
         
         dbus_message_iter_init_append(results, &iter);
         
-        /* Number of results */
-        dbus_int32_t number_of_results = g_slist_length(scan_results_save);
-        
         if (number_of_results > WLANCOND_MAX_NETWORKS) {
                 DLOG_DEBUG("Limiting result %d to %d", number_of_results, 
                            WLANCOND_MAX_NETWORKS);
@@ -240,7 +238,7 @@ static void connected_signal(unsigned char* bssid, dbus_int32_t auth_status)
 static void handle_wap_event(struct scan_results_t *scan_results, 
                              struct iw_event *event) 
 {
-        int sock, state, scan_state;
+        int state, scan_state;
         gboolean zero_address = FALSE;
 
         // Spurious event
@@ -277,19 +275,6 @@ static void handle_wap_event(struct scan_results_t *scan_results,
                 
                 state = get_wlan_state();
                 
-                if (get_wpa_mode() == FALSE && state != WLAN_CONNECTED &&
-                    get_mode() == WLANCOND_INFRA) {
-                        /* Let's wait until DCHP has finished until 
-                           full WLAN power save */
-                        if ((sock = socket_open()) >= 0) {
-                                if (set_power_state(WLANCOND_LONG_CAM,
-                                                    sock) == FALSE) {
-                                        DLOG_ERR("Failed to set power save");
-                                }
-                                close (sock);
-                        }
-                }
-                
                 connected_signal(event->u.ap_addr.sa_data, auth_status);
                 // If not roaming, set NO_ADDRESS state
                 if (state != WLAN_CONNECTED) {
@@ -331,7 +316,10 @@ int print_event_token(struct iw_event *event,
                             memcpy(scan_results->ssid, event->u.essid.pointer, 
                                    len);
                     }
-                    // Len includes null termination
+                    
+                    // Keep the API the same i.e. add Null termination
+                    len++;
+
                     scan_results->ssid_len = len;
                     
                     if (event->u.essid.flags)
@@ -502,7 +490,6 @@ int print_event_token(struct iw_event *event,
 static int handle_wpa_ie_assoc_event_binary(unsigned char* p, 
                                             unsigned int length) 
 {
-        gboolean own_ie = FALSE;
         int status = 0;
         int ie_len;
         int sock;
@@ -515,15 +502,12 @@ static int handle_wpa_ie_assoc_event_binary(unsigned char* p,
                 DLOG_DEBUG("Invalid length: %d", length);
                 return -1;
         }
+
+        ie_len = length - ETH_ALEN - 1;
         
         if (memcmp(p, own_mac, ETH_ALEN) == 0) 
         {
                 //DLOG_DEBUG("Own WPA IE found");
-                own_ie = TRUE;
-        }
-        ie_len = length - ETH_ALEN - 1;
-        
-        if (own_ie) {
                 update_own_ie(g_memdup(p + ETH_ALEN + 1, ie_len), ie_len);
         } else {
                 /* Push WPA IEs to security SW */
@@ -532,11 +516,8 @@ static int handle_wpa_ie_assoc_event_binary(unsigned char* p,
                 /* Check if we are roaming */
                 if (get_wlan_state() == WLAN_CONNECTED ||
                     get_wlan_state() == WLAN_NO_ADDRESS) {
-                        if ((sock = socket_open()) >= 0) {
-                                set_power_state(WLANCOND_POWER_ON,
-                                                sock);
-                                close (sock);
-                        }
+                        sock = socket_open();
+                        set_power_state(WLANCOND_POWER_ON, sock);
                 }
         } 
                 
@@ -600,6 +581,11 @@ static int handle_wpa_ie_event_binary(unsigned char* p, unsigned int length,
         if (ap_info.pairwise_cipher & CIPHER_SUITE_WEP40 || 
             ap_info.pairwise_cipher & CIPHER_SUITE_WEP104) {
                 DLOG_DEBUG("WEP supported for unicast");
+                
+                if (no_wep == TRUE) {
+                        DLOG_DEBUG("In WPA mode WEP is not allowed");
+                        scan_results->cap_bits |= WLANCOND_UNSUPPORTED_NETWORK;
+                }
         }
         /* Group */
         if (ap_info.group_cipher & CIPHER_SUITE_CCMP) {
@@ -613,6 +599,11 @@ static int handle_wpa_ie_event_binary(unsigned char* p, unsigned int length,
         if (ap_info.group_cipher & CIPHER_SUITE_WEP40 || 
             ap_info.group_cipher & CIPHER_SUITE_WEP104) {
                 DLOG_DEBUG("WEP supported for multicast");
+
+                if (no_wep == TRUE) {
+                        DLOG_DEBUG("In WPA mode WEP is not allowed");
+                        scan_results->cap_bits |= WLANCOND_UNSUPPORTED_NETWORK;
+                }
         }
         /* Remove WEP bit to make UI show correct dialogs */
         if (no_wep) {
@@ -686,8 +677,8 @@ static inline int index2name(int skfd, int ifindex, char *name)
  */
 struct wireless_iface *get_interface_data(int ifindex)
 {
-        struct wireless_iface *	curr;
-        int skfd = -1;	/* ioctl socket */
+        struct wireless_iface *curr;
+        int skfd;
         
         /* Search for it in the database */
         curr = interface_cache;
@@ -706,10 +697,7 @@ struct wireless_iface *get_interface_data(int ifindex)
                 curr = curr->next;
         }
         
-        if((skfd = socket_open()) < 0)
-        {
-                return(NULL);
-        }
+        skfd = socket_open();
         
         curr = g_new(struct wireless_iface, 1);
         
@@ -720,14 +708,9 @@ struct wireless_iface *get_interface_data(int ifindex)
         {
                 perror("index2name");
                 g_free(curr);
-                close(skfd);
                 return(NULL);
         }
         curr->has_range = (iw_get_range_info(skfd, curr->ifname, &curr->range) >= 0);
-
-        /* Done */
-        close(skfd);
-        
         /* Link it */
         curr->next = interface_cache;
         interface_cache = curr;
