@@ -187,7 +187,11 @@ gboolean add_mode_listener(DBusConnection *connection) {
 
 #ifdef ACTIVITY_CHECK
         dbus_bus_add_match(connection, 
-                           "interface=" MCE_SIGNAL_IF, NULL);
+                           "interface=" MCE_SIGNAL_IF
+                           ",member=" MCE_INACTIVITY_SIG, NULL);
+        dbus_bus_add_match(connection, 
+                           "interface=" MCE_SIGNAL_IF
+                           ",member=" MCE_DEVICE_MODE_SIG, NULL);
 #else
         dbus_bus_add_match(connection, 
                            "interface=" MCE_SIGNAL_IF 
