@@ -49,14 +49,14 @@ static guint mic_failure_running_timer_id = 0;
 static const guint8 WPA_CIPHER_SUITE_NONE[] = {0x00, 0x50, 0xf2, 0};
 static const guint8 WPA_CIPHER_SUITE_WEP40[] = {0x00, 0x50, 0xf2, 1};
 static const guint8 WPA_CIPHER_SUITE_TKIP[] = {0x00, 0x50, 0xf2, 2};
-static const guint8 WPA_CIPHER_SUITE_WRAP[] = {0x00, 0x50, 0xf2, 3};
+//static const guint8 WPA_CIPHER_SUITE_WRAP[] = {0x00, 0x50, 0xf2, 3};
 static const guint8 WPA_CIPHER_SUITE_CCMP[] = {0x00, 0x50, 0xf2, 4};
 static const guint8 WPA_CIPHER_SUITE_WEP104[] = {0x00, 0x50, 0xf2, 5};
 
 static const guint8 RSN_CIPHER_SUITE_NONE[] = {0x00, 0x0f, 0xac, 0};
 static const guint8 RSN_CIPHER_SUITE_WEP40[] = {0x00, 0x0f, 0xac, 1};
 static const guint8 RSN_CIPHER_SUITE_TKIP[] = {0x00, 0x0f, 0xac, 2};
-static const guint8 RSN_CIPHER_SUITE_WRAP[] = {0x00, 0x0f, 0xac, 3};
+//static const guint8 RSN_CIPHER_SUITE_WRAP[] = {0x00, 0x0f, 0xac, 3};
 static const guint8 RSN_CIPHER_SUITE_CCMP[] = {0x00, 0x0f, 0xac, 4};
 static const guint8 RSN_CIPHER_SUITE_WEP104[] = {0x00, 0x0f, 0xac, 5};
 
@@ -66,18 +66,16 @@ static const guint8 RSN_KEY_MGMT_PSK[] = {0x00, 0x0f, 0xac, 2};
 static const guint8 WPA_KEY_MGMT_802_1X[] = {0x00, 0x50, 0xf2, 1};
 static const guint8 WPA_KEY_MGMT_PSK[] = {0x00, 0x50, 0xf2, 2};
 
-static const guint8 WPA_OUI[] = { 0x00, 0x50, 0xf2 };
-
 int parse_rsn_cipher_suite(guint8 *suite)
 {
-	if (memcmp(suite, RSN_CIPHER_SUITE_NONE, CIPHER_SUITE_LEN) == 0)
-		return CIPHER_SUITE_NONE;
-	if (memcmp(suite, RSN_CIPHER_SUITE_WEP40, CIPHER_SUITE_LEN) == 0)
-		return CIPHER_SUITE_WEP40;
 	if (memcmp(suite, RSN_CIPHER_SUITE_TKIP, CIPHER_SUITE_LEN) == 0)
 		return CIPHER_SUITE_TKIP;
 	if (memcmp(suite, RSN_CIPHER_SUITE_CCMP, CIPHER_SUITE_LEN) == 0)
 		return CIPHER_SUITE_CCMP;
+	if (memcmp(suite, RSN_CIPHER_SUITE_NONE, CIPHER_SUITE_LEN) == 0)
+		return CIPHER_SUITE_NONE;
+	if (memcmp(suite, RSN_CIPHER_SUITE_WEP40, CIPHER_SUITE_LEN) == 0)
+		return CIPHER_SUITE_WEP40;
 	if (memcmp(suite, RSN_CIPHER_SUITE_WEP104, CIPHER_SUITE_LEN) == 0)
 		return CIPHER_SUITE_WEP104;
 	return 0;
@@ -94,16 +92,16 @@ int parse_rsn_key_mgmt_suite(guint8 *suite)
 
 int parse_wpa_cipher_suite(guint8* suite)
 {
+	if (memcmp(suite, WPA_CIPHER_SUITE_TKIP, CIPHER_SUITE_LEN) == 0)
+		return CIPHER_SUITE_TKIP;
+	if (memcmp(suite, WPA_CIPHER_SUITE_CCMP, CIPHER_SUITE_LEN) == 0)
+		return CIPHER_SUITE_CCMP;
 	if (memcmp(suite, WPA_CIPHER_SUITE_NONE, CIPHER_SUITE_LEN) == 0)
 		return CIPHER_SUITE_NONE;
 	if (memcmp(suite, WPA_CIPHER_SUITE_WEP40, CIPHER_SUITE_LEN) == 0)
 		return CIPHER_SUITE_WEP40;
 	if (memcmp(suite, WPA_CIPHER_SUITE_WEP104, CIPHER_SUITE_LEN) == 0)
 		return CIPHER_SUITE_WEP104;
-	if (memcmp(suite, WPA_CIPHER_SUITE_TKIP, CIPHER_SUITE_LEN) == 0)
-		return CIPHER_SUITE_TKIP;
-	if (memcmp(suite, WPA_CIPHER_SUITE_CCMP, CIPHER_SUITE_LEN) == 0)
-		return CIPHER_SUITE_CCMP;
 	return 0;
 }
 
@@ -147,18 +145,13 @@ gboolean set_encryption_method(guint32 cipher)
                     return FALSE;
 	}
 
-        if ((sock = socket_open()) < 0)
-        {
-                return FALSE;
-        }        
+        sock = socket_open();
         
 	if (ioctl(sock, SM_DRV_WPA_SET_WPA, &req) < 0) {
                 DLOG_ERR("Could not set WPA mode");
-                close(sock);
                 return FALSE;
 	}
-        
-        close(sock);
+
         return TRUE;
 }
 
@@ -358,10 +351,9 @@ int parse_wpa_ie(unsigned char* wpa_ie, unsigned int wpa_ie_len,
 {
         struct wpa_ie_t *wpa_hdr;
         guint8 *hdr_p;
-        guint ind;
-        guint i;
+        guint ind, i;
         guint cipher_count = 0;
-        guint8 WPA1_OUI[] = { 0x00, 0x50, 0xf2, 1 };
+        const guint8 WPA1_OUI[] = { 0x00, 0x50, 0xf2, 1 };
         
         /* Do data checking, we have to make sure all the time that we
            don't go past the IE length , index variable counts the 
